Unit fMain;

{$mode objfpc}{$H+}

(* Copyright (c) 2024 by Markus Müller / www.elela.de
   This code is free for use, without any warranty.
*)

Interface

Uses
  Classes, SysUtils, Forms, Controls, Graphics, Dialogs, ExtCtrls, StdCtrls,
  IniFiles, fpjson, jsonparser,
  httpsend, ssl_openssl3_lib, ssl_openssl3;

Type

  { TfrmMain }

  TfrmMain = Class(TForm)
    btnQuery: TButton;
    btnGetTokenSynapse: TButton;
    edClientID: TEdit;
    edClientSecret: TEdit;
    Label1: TLabel;
    Label2: TLabel;
    edAccessToken: TMemo;
    Label3: TLabel;
    Label4: TLabel;
    memAnswer: TMemo;
    memQuery: TMemo;
    Panel1: TPanel;
    Panel2: TPanel;
    Splitter1: TSplitter;
    Procedure btnGetTokenSynapseClick(Sender: TObject);
    Procedure btnQueryClick(Sender: TObject);
    Procedure FormClose(Sender: TObject; Var CloseAction: TCloseAction);
    Procedure FormCreate(Sender: TObject);
  private
    procedure WriteStrToStream2(const Stream: TStream; Value: AnsiString);
  public
  End;

Var
  frmMain: TfrmMain;

Implementation

{$R *.lfm}

{ TfrmMain }

Procedure TfrmMain.FormCreate(Sender: TObject);
Var ini: TIniFile;
Begin
  ini := TIniFile.Create(ChangeFileExt(Application.ExeName, '.ini'));
  edClientID.Text := ini.ReadString('Access', 'ClientID', '<ClientID>');
  edClientSecret.Text := ini.ReadString('Access', 'ClientSecret', '<ClientSecret>');
  FreeAndNil(ini);
  If FileExists('Query.txt') Then
    memQuery.Lines.LoadFromFile('Query.txt');
end;

Procedure TfrmMain.FormClose(Sender: TObject; Var CloseAction: TCloseAction);
Var ini: TIniFile;
Begin
  CloseAction := caFree;
  ini := TIniFile.Create(ChangeFileExt(Application.ExeName, '.ini'));
  ini.WriteString('Access', 'ClientID', edClientID.Text);
  ini.WriteString('Access', 'ClientSecret', edClientSecret.Text);
  ini.UpdateFile;
  FreeAndNil(ini);
  If FileExists('Query.txt') Then
    DeleteFile('Query.txt');
  memQuery.Lines.SaveToFile('Query.txt');
end;

procedure TfrmMain.WriteStrToStream2(const Stream: TStream; Value: AnsiString);
{$IFDEF CIL}
var
  buf: Array of Byte;
{$ENDIF}
begin
{$IFDEF CIL}
  buf := BytesOf(Value);
  Stream.Write(buf,length(Value));
{$ELSE}
  Stream.Write(PAnsiChar(Value)^, Length(Value));
{$ENDIF}
end;

Procedure TfrmMain.btnGetTokenSynapseClick(Sender: TObject);
Var HTTP: THTTPSend;
  URLData: string;
  Result: Boolean;
  J, J2: TJSONData;
Begin
  HTTP := THTTPSend.Create;
  HTTP.Headers.Add('accept: */*');
  HTTP.UserAgent := 'curl/7.81.0';
  HTTP.MimeType := 'application/x-www-form-urlencoded';
  URLData := 'grant_type=client_credentials&client_id=' + edClientID.Text + '&client_secret=' + edClientSecret.Text + '&scope=supply.domain';
  WriteStrToStream2(HTTP.Document, URLData);
  Result := HTTP.HTTPMethod('POST', 'https://identity.nexar.com/connect/token');
  if Result then
  Begin
    HTTP.Document.Position := 0;
    J := GetJSON(HTTP.Document);
    If Assigned(J) Then
    Begin
      If J.JSONType = jtObject Then
      Begin
        J2 := J.FindPath('access_token');
        If Assigned(J2) Then
          edAccessToken.Text := J2.AsString;
      End;
    End;
  End;
  FreeAndNil(HTTP);
end;

Procedure TfrmMain.btnQueryClick(Sender: TObject);
Var HTTP: THTTPSend;
  s, URLData: string;
  Result: Boolean;
  J: TJSONObject;
Begin
  s := Trim(edAccessToken.Text);
  If s = '' Then
  Begin
    btnGetTokenSynapseClick(Nil);
    s := Trim(edAccessToken.Text);
    If s = '' Then Exit;
  End;
  HTTP := THTTPSend.Create;
  HTTP.Headers.Add('accept: */*');
  HTTP.Headers.Add('Authorization: Bearer ' + s);
  HTTP.UserAgent := 'curl/7.81.0';
  HTTP.MimeType := 'application/json';
  J := TJSONObject.Create;
  J.Add('query', memQuery.Text);
  URLData := J.AsJSON;
  FreeAndNil(J);
  WriteStrToStream2(HTTP.Document, URLData);
  Result := HTTP.HTTPMethod('POST', 'https://api.nexar.com/graphql');
  if Result then
  Begin
    HTTP.Document.Position := 0;
    memAnswer.Lines.LoadFromStream(HTTP.Document);
  End;
  FreeAndNil(HTTP);
End;

End.

